package Langertha::Knarr::Output::Tools;
our $VERSION = '0.006';
# ABSTRACT: Primary tool-output helpers for Knarr
use strict;
use warnings;
use parent 'Langertha::Output::Tools';

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Langertha::Knarr::Output::Tools - Primary tool-output helpers for Knarr

=head1 VERSION

version 0.006

=head1 SUPPORT

=head2 Issues

Please report bugs and feature requests on GitHub at
L<https://github.com/Getty/langertha-knarr/issues>.

=head1 CONTRIBUTING

Contributions are welcome! Please fork the repository and submit a pull request.

=head1 AUTHOR

Torsten Raudssus <torsten@raudssus.de> L<https://raudssus.de/>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2026 by Torsten Raudssus.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
